//sound.fruit: a soundclip is taken and translated into a 3d form. 

import ddf.minim.analysis.*;
import ddf.minim.*;
import processing.dxf.*;


//parameters
int m = 6; //logavg 1 controls distribution of spectrum
int n = 6; //logavg 2 controls amplitude of spectrum
int ptDensity = 200; //density of circle around y-axis
float beta = 0; //angle
int idx = 0; //index
float r; //radius
float sc = 1; //object scale

//sound setup
Minim       minim;
FFT         fft;
AudioInput in;
AudioPlayer thinking;

//sphere setup
PVector [][] coords;

//setup export dxf object record button
RecordButton recIt;
int recX, recY, recSize;
boolean recHit = false;
boolean record = false;
color recFill, recHighlight;
int waveSize = 0;
boolean waveHit = false;
int waveFade = 255;

//play button setup
PlayButton playIt;
int playX, playY, playSize;
boolean playHit = false; 
color playFill, playHighlight;
boolean play = true;

//scrollbar setup

HScrollbar hs1, hs2;  // Two scrollbars





//===================================================================

void setup() {

  size(displayWidth, (displayHeight-20), P3D);


  //sound creation
  minim = new Minim(this);
  thinking = minim.loadFile("punches.mp3", 1024);
  thinking.loop();
  fft = new FFT(thinking.bufferSize(), thinking.sampleRate());
  fft.logAverages(m, n); 

  //sphere geometry calculations
  r = width/50; //radius
  beta = TWO_PI/ptDensity; //y-axis rotation
  coords = new PVector [ptDensity+1][fft.avgSize()];
  float x, y, z;
  for (int i = 0; i<coords.length; i++) {
    float u = (i*TWO_PI/ptDensity);
    for (int j = 0; j<coords[i].length; j++) {
      float v = (PI/coords[i].length)*j-PI/2;
      //      x = r * cos(theta);
      //      y = r * sin(theta);
      x=r* sin(u)* cos(v); 
      y=r * cos(u) * cos(v); 
      z= r * sin(v);

      coords[i][j] = new PVector(x, y, z);
      println(u + " " + v + " " + coords[i][j]);
    }
  }

  //playbutton parameters
  playX = width-100; 
  playY = height-100;
  playSize = 30;
  ellipseMode(CENTER);
  playFill = color(5);
  playHighlight = color(200);
  //create play button
  playIt = new PlayButton();

  //recbutton parameters
  recX = width-60; 
  recY = height-100;
  recSize = 30;
  //ellipseMode(CENTER);
  recFill = color(5);
  recHighlight = color(200);
  //create play button
  recIt = new RecordButton();
}

//=========================================================================

void draw() {
  lights();
  background(5);


  //play button
  stroke(250);
  updatePlay(mouseX, mouseY);
  if (playHit) {
    fill(playHighlight);
  } else {
    fill(playFill);
  }
  playIt.display();
  //  button graphics
  fill(255);
  if (play == true) {

    noStroke();
    beginShape();
    vertex((playX+playSize/20*7), playY);
    vertex((playX-playSize/20*5), (playY+playSize/20*7));
    vertex((playX-playSize/20*5), (playY-playSize/20*7));
    endShape(CLOSE);
  }
  if (!play == true) {
    rectMode(CENTER);

    noStroke();
    rect(playX-3, playY, 4, 13);
    rect(playX+3, playY, 4, 13);
  }



  //record button
  stroke(250);
  updateRec(mouseX, mouseY);
  if (recHit) {
    fill(recHighlight);
  } else {
    fill(recFill);
  }
  recIt.display();
  //  button graphics
  if (play == true) {
    fill(255);
  }
  if (!play == true) {
    rectMode(CENTER);
    fill(255);
  }
  //start recording to file
  if (record == true) {
    beginRaw(DXF, "soundform##.dxf");
  }
  //record wave
  if (waveHit == true) {
    noFill();
    stroke(250, 10, 10, waveFade);
    ellipse(recX, recY, waveSize, waveSize);
    waveSize += 20;
    waveFade-= 12;
  }



  pushMatrix();
  
  translate(width/2, height/2, width/3);
  scale(sc);
  stroke(250);
  strokeWeight(1);
  //noFill();
  //noStroke();
  color(200, 200, 200);

  //setup view
  //  rotateX(207);
  rotateX(0.01*mouseY);
  rotateY(0.01*mouseX);

  //call input

  fft.forward(thinking.mix);





  //advance index
  idx = (idx+1)%(ptDensity);

  // calculate current row
  for (int i = 0; i<fft.avgSize (); i++) { //this loop draws the half circle

    float theta = (PI/fft.avgSize())*i-PI/2;


    //change mag
    float add = (coords[idx][i].mag() + (0.05*(r+modFactor(i)*fft.getAvg(i))));

    coords[idx][i].setMag(add);
  }

  //draw sphere

  for (int j = 0; j< (ptDensity); j++) { //loop rotates half circle around y-axis to form sphere



    beginShape(QUAD_STRIP);
    for (int i = 0; i<fft.avgSize (); i++) { //this loop draws the half circle

      //      float theta = (PI/fft.avgSize())*i-PI/2;
      noStroke();
      //       stroke(map(fft.getAvg(i), 0, fft.avgSize(), 0, 256), 50, 55);
      fill(map(fft.getAvg(i), 0, fft.avgSize(), 0, 256), 50, 55);

      vertex(coords[j][i].x, coords[j][i].y, coords[j][i].z);

      if (j == ptDensity-1) {
        vertex(coords[0][i].x, coords[0][i].y, coords[0][i].z);
      } else {
        vertex(coords[j+1][i].x, coords[j+1][i].y, coords[j+1][i].z);
      } //create quadstrip-compatible geometry
    }
    endShape(CLOSE);
  }

  popMatrix();
  //button





  //stop recording
  if (record == true) {
    endRaw();
    record = false;
  }

  //stop record wave
  if (waveSize > 1000) {
    waveHit = false;
    waveSize = 0; 
    waveFade = 255;
  }
}


//===================================================================

float modFactor(float i) {
  //  float a = mouseX/(float)width;
  //  float b = mouseY/(float)height;
  float a = .06;
  float b = .2;
  if (mousePressed)
    println(a + " " + b);
  return a*i+b;
}


//===================================================================
//modification object scale 

void keyTyped() {
  if (key == 'n' || key == 'N') {
    sc -= 0.1;
  }
  else if (key == 'm' || key == 'M') {
    sc += 0.1;
  }
}

